'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var prefixAll = _interopDefault(require('inline-style-prefixer/static'));
var test = _interopDefault(require('tape'));

var uppercasePattern = /[A-Z]/g;
var msPattern = /^ms-/;
var cache = {};

function hyphenateStyleName(prop) {
  return prop in cache ? cache[prop] : cache[prop] = prop.replace(uppercasePattern, '-$&').toLowerCase().replace(msPattern, '-ms-');
}

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function injectStyle(styletron, styles, media, pseudo) {
  var classString = '';
  for (var key in styles) {
    var val = styles[key];
    var valType = typeof val === 'undefined' ? 'undefined' : _typeof(val);
    if (valType === 'string' || valType === 'number') {
      classString += ' ' + styletron.injectRawDeclaration({
        block: hyphenateStyleName(key) + ':' + val,
        media: media,
        pseudo: pseudo
      });
      continue;
    }
    if (Array.isArray(val)) {
      if (val.length === 0) {
        continue;
      }
      var hyphenated = hyphenateStyleName(key);
      var block = hyphenated + ':' + val[0];
      for (var i = 1; i < val.length; i++) {
        block += ';' + hyphenated + ':' + val[i];
      }
      classString += ' ' + styletron.injectRawDeclaration({
        block: block,
        media: media,
        pseudo: pseudo
      });
      continue;
    }
    if (valType === 'object') {
      if (key[0] === ':') {
        classString += ' ' + injectStyle(styletron, val, media, key);
        continue;
      }
      if (key.substring(0, 6) === '@media') {
        classString += ' ' + injectStyle(styletron, val, key.substr(7), pseudo);
        continue;
      }
    }
  }
  // remove leading space on way out
  return classString.slice(1);
}

var _typeof$1 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var prefixedBlockCache = {};

function injectStylePrefixed(styletron, styles, media, pseudo) {
  var cache = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : prefixedBlockCache;

  var classString = '';
  for (var originalKey in styles) {
    var originalVal = styles[originalKey];
    var originalValType = typeof originalVal === 'undefined' ? 'undefined' : _typeof$1(originalVal);
    var isPrimitiveVal = originalValType === 'string' || originalValType === 'number';
    if (isPrimitiveVal || Array.isArray(originalVal)) {
      var block = '';
      if (isPrimitiveVal && cache.hasOwnProperty(originalKey) && cache[originalKey].hasOwnProperty(originalVal)) {
        block = cache[originalKey][originalVal];
      } else {
        var _prefixAll;

        var prefixed = prefixAll((_prefixAll = {}, _prefixAll[originalKey] = originalVal, _prefixAll));
        for (var prefixedKey in prefixed) {
          var prefixedVal = prefixed[prefixedKey];
          var prefixedValType = typeof prefixedVal === 'undefined' ? 'undefined' : _typeof$1(prefixedVal);
          if (prefixedValType === 'string' || prefixedValType === 'number') {
            block += hyphenateStyleName(prefixedKey) + ':' + prefixedVal + ';';
            continue;
          }
          if (Array.isArray(prefixedVal)) {
            var hyphenated = hyphenateStyleName(prefixedKey);
            for (var i = 0; i < prefixedVal.length; i++) {
              block += hyphenated + ':' + prefixedVal[i] + ';';
            }
            continue;
          }
        }
        block = block.slice(0, -1); // Remove trailing semicolon
        if (isPrimitiveVal) {
          if (!cache.hasOwnProperty(originalKey)) {
            cache[originalKey] = {};
          }
          cache[originalKey][originalVal] = block;
        }
      }
      classString += ' ' + styletron.injectRawDeclaration({
        block: block,
        media: media,
        pseudo: pseudo
      });
    }
    if (originalValType === 'object') {
      if (originalKey[0] === ':') {
        classString += ' ' + injectStylePrefixed(styletron, originalVal, media, originalKey, cache);
        continue;
      }
      if (originalKey.substring(0, 6) === '@media') {
        classString += ' ' + injectStylePrefixed(styletron, originalVal, originalKey.substr(7), pseudo, cache);
        continue;
      }
    }
  }
  // remove leading space on way out
  return classString.slice(1);
}

test('test injection', function (t) {
  var decls = [];
  var spy = {
    injectRawDeclaration: function injectRawDeclaration(decl) {
      decls.push(decl);
      return decls.length;
    }
  };
  var classString = injectStyle(spy, {
    color: 'red',
    backgroundColor: 'blue',
    '@media (max-width: 500px)': {
      color: 'purple'
    },
    ':hover': {
      background: 'orange'
    }
  });
  t.equal(classString, '1 2 3 4');
  t.deepEqual(decls, [{ block: 'color:red', media: undefined, pseudo: undefined }, {
    block: 'background-color:blue',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'color:purple',
    media: '(max-width: 500px)',
    pseudo: undefined
  }, { block: 'background:orange', media: undefined, pseudo: ':hover' }]);
  t.end();
});

test('test injection array', function (t) {
  var decls = [];
  var spy = {
    injectRawDeclaration: function injectRawDeclaration(decl) {
      decls.push(decl);
      return decls.length;
    }
  };
  var classString = injectStyle(spy, {
    color: ['red', 'blue'],
    '@media (max-width: 500px)': {
      color: ['purple', 'orange']
    },
    ':hover': {
      color: ['green', 'yellow']
    }
  });
  t.equal(classString, '1 2 3');
  t.deepEqual(decls, [{ block: 'color:red;color:blue', media: undefined, pseudo: undefined }, {
    block: 'color:purple;color:orange',
    media: '(max-width: 500px)',
    pseudo: undefined
  }, { block: 'color:green;color:yellow', media: undefined, pseudo: ':hover' }]);
  t.end();
});

test('test injection prefixed', function (t) {
  var decls = [];
  var spy = {
    injectRawDeclaration: function injectRawDeclaration(decl) {
      decls.push(decl);
      return decls.length;
    }
  };
  var classString = injectStylePrefixed(spy, {
    display: 'flex',
    height: ['min-content', '100%'],
    transition: 'height 1s',
    ':hover': {
      backgroundColor: 'linear-gradient(to bottom, red, green)'
    },
    '@media (max-width: 500px)': {
      flexGrow: 1
    }
  });
  t.equal(classString, '1 2 3 4 5');
  t.deepEqual(decls, [{
    block: 'display:-webkit-box;display:-moz-box;display:-ms-flexbox;display:-webkit-flex;display:flex',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'height:-webkit-min-content;height:-moz-min-content;height:min-content;height:100%',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'transition:height 1s;-webkit-transition:height 1s;-moz-transition:height 1s',
    media: undefined,
    pseudo: undefined
  }, {
    block: 'background-color:-webkit-linear-gradient(to bottom, red, green);background-color:-moz-linear-gradient(to bottom, red, green);background-color:linear-gradient(to bottom, red, green)',
    media: undefined,
    pseudo: ':hover'
  }, {
    block: 'flex-grow:1;-webkit-flex-grow:1',
    media: '(max-width: 500px)',
    pseudo: undefined
  }]);
  t.end();
});

test('test prefixed cache', function (t) {
  var decls = [];
  var spy = {
    injectRawDeclaration: function injectRawDeclaration(decl) {
      decls.push(decl);
      return decls.length;
    }
  };
  var cache = { flexGrow: { 1: 'color:red' } };
  injectStylePrefixed(spy, {
    display: 'flex',
    flexGrow: 1
  }, undefined, undefined, cache);
  t.deepEqual(cache, {
    display: {
      flex: 'display:-webkit-box;display:-moz-box;display:-ms-flexbox;display:-webkit-flex;display:flex'
    },
    flexGrow: { 1: 'color:red' }
  });
  t.deepEqual(decls, [{
    block: 'display:-webkit-box;display:-moz-box;display:-ms-flexbox;display:-webkit-flex;display:flex',
    media: undefined,
    pseudo: undefined
  }, { block: 'color:red', media: undefined, pseudo: undefined }]);
  t.end();
});
//# sourceMappingURL=browser.js.map
